import unittest

from mock import patch

from yolapy.models import siteimport


class SiteImportTestCase(unittest.TestCase):
    def setUp(self):
        patcher = patch.object(siteimport, 'Yola', autospec=True)
        self.yola = patcher.start().return_value
        self.addCleanup(patcher.stop)


class SiteImportCreate(SiteImportTestCase):
    """SiteImport.create"""
    def setUp(self):
        super(SiteImportCreate, self).setUp()
        self.url = 'url'
        self.user_id = 'user_id'
        self.siteimport = siteimport.SiteImport.create(self.url, self.user_id)

    def test_creates_site_import_with_passed_arguments(self):
        self.yola.create_site_import.assert_called_once_with(
            url=self.url, user_id=self.user_id)

    def test_returns_site_import_instance(self):
        self.assertIsInstance(self.siteimport, siteimport.SiteImport)


class SiteImportGet(SiteImportTestCase):
    def setUp(self):
        super(SiteImportGet, self).setUp()
        self.id = 'id'
        self.site_import_data = {'id': self.id}
        self.yola.get_site_import.return_value = self.site_import_data
        self.site_import = siteimport.SiteImport.get(self.id)

    def test_gets_specified_site_import(self):
        self.yola.get_site_import.assert_called_once_with(self.id)

    def test_returns_site_import_instance(self):
        self.assertIsInstance(self.site_import, siteimport.SiteImport)


class SiteImportList(SiteImportTestCase):
    def setUp(self):
        super(SiteImportList, self).setUp()
        self.site_imports_data = [{'id': 1}, {'id': 2}]
        self.yola.list_site_imports.return_value = {
            'results': self.site_imports_data}
        self.filters = {'one': 1, 'two': 2}
        self.site_imports = siteimport.SiteImport.list(**self.filters)

    def test_gets_specified_site_import_list(self):
        _, kwargs = self.yola.list_site_imports.call_args
        for filter, value in self.filters.items():
            self.assertEqual(kwargs[filter], value)

    def test_returns_site_import_list(self):
        for site_import in self.site_imports:
            self.assertIsInstance(site_import, siteimport.SiteImport)



import demands
from uuid import uuid4

from tests.test_integration.helpers import (
    create_user, create_site, create_user_with_subscription)
from tests.test_integration.test_case import YolaServiceTestCase


class TestYolaUser(YolaServiceTestCase):
    """Yola: User resource"""

    @classmethod
    def setUpClass(cls):
        super(TestYolaUser, cls).setUpClass()
        cls.user = cls._create_user()
        cls.user_id = cls.user['id']
        cls.site = create_site(cls.service, cls.user_id)

    @classmethod
    def tearDownClass(cls):
        super(TestYolaUser, cls).tearDownClass()
        cls.service.delete_site(cls.site['id'])
        cls.service.delete_user(cls.user_id)

    @classmethod
    def _create_user(cls, **custom_attrs):
        custom_attrs.setdefault('name', 'John')
        return create_user_with_subscription(cls.service, **custom_attrs)

    def test_create_user(self):
        self.assertEqual(self.user['name'], 'John')

    def test_update_user(self):
        user = self._create_user(name='Original Name')
        try:
            user = self.service.update_user(user['id'], name='New Name')
            self.assertEqual(user['name'], 'New Name')
        finally:
            self.service.delete_user(user['id'])

    def test_get_user(self):
        user = self.service.get_user(self.user_id)
        self.assertEqual(user['name'], 'John')

    def test_list_users(self):
        response = self.service.list_users(page_size=1)
        self.assertEqual(len(response['results']), 1)

    def test_request_filtered_user_list(self):
        response = self.service.list_users(id=self.user_id)
        self.assertEqual(len(response['results']), 1)
        self.assertEqual(response['results'][0]['id'], self.user_id)

    def test_delete_user(self):
        user = self._create_user()
        self.service.delete_user(user['id'])
        with self.assertRaises(demands.HTTPServiceError):
            self.service.get_user(user['id'])

    def test_get_sso_create_site_url(self):
        url = self.service.get_sso_create_site_url(self.user_id, 'example.com')
        self.assertTrue(url.startswith('http'))

    def test_get_sso_open_site_url(self):
        url = self.service.get_sso_open_site_url(self.user_id)
        self.assertTrue(url.startswith('http'))

    def test_get_sso_open_site_url_with_site_id(self):
        url = self.service.get_sso_open_site_url(
            self.user_id, site_id=self.site['id'])
        self.assertIn(self.site['id'], url)

    def test_get_user_wsites(self):
        expected_keys = (
            'id', 'owner_id', 'name', 'template_slug', 'created_at',
            'deleted_at', 'updated_at'
        )

        ws_user = create_user(
            self.service, is_ws=True,
            site_url='https://{}.yolasite.com'.format(uuid4().hex)
        )
        try:
            user_wsites = self.service.get_user_wsites(ws_user['id'])
            self.assertEqual(len(user_wsites), 1)
            self.assertEqual(user_wsites[0]['owner_id'], ws_user['id'])
            self.assertItemsEqual(user_wsites[0].keys(), expected_keys)
        finally:
            self.service.delete_user(ws_user['id'])

    def test_set_site_url(self):
        user = create_user(
            self.service,
            is_ws=True,
            site_url='https://{}.yolasite.com'.format(uuid4().hex),
        )
        try:
            new_site_url = 'https://{}.site.com'.format(uuid4().hex)
            user = self.service.set_site_url(user['id'], new_site_url)
            self.assertEqual(new_site_url, user['preferences']['site_url'])
        finally:
            self.service.delete_user(user['id'])



class UserResourceMixin(object):
    """Methods for managing User resources."""

    def create_user(self, **attributes):
        """Create a user.

        >>> user = yola.create_user(
            name='John',
            surname='Smith',
            email='johnsmith@example.com',
            partner_id='WL_PARTNER_ID',
            preferences={'preference_name': 'preference_value'})
        >>> user['name']
        'John'
        """
        response = self.post(self._user_path(), json=attributes).json()
        response['signup_date'] = response.pop('signupDate')
        return response

    def update_user(self, user_id, **attributes):
        """Update a user.

        >>> yola.update_user('user_id', name='New name')
        """
        return self.patch(self._user_path(user_id), json=attributes).json()

    def get_user(self, user_id):
        """Get details for a particular user.

        >>> user = yola.get_user('user_id')
        >>> user['name']
        'John'
        """
        response = self.get(self._user_path(user_id)).json()
        response['signup_date'] = response.pop('signupDate')
        return response

    def list_users(self, **filters):
        """Return paginated list of users.

        >>> yola.list_users()
        {
            'count': 999,
            'previous': None,
            'next': 'https://wl.qa.yola.net/pr/users/?page=2',
            'results': [
                {'name': 'John', 'surname': 'Smith', ...}
            ]
        }

        If there are no users, ``results`` will be an empty list. No exception
        will be raised.

        You may pass pagination options and attribute filters as keyword
        arguments.  See https://wl.qa.yola.net/users/ for available parameters.

        For example:

        >>> yola.list_users(page=2, page_size=50, partner_id='WL_YOLA')
        """
        return self.get(self._user_path(), params=filters).json()

    def delete_user(self, user_id):
        """Delete a user.

        >>> yola.delete_user('user_id')
        """
        self.delete(self._user_path(user_id))

    def get_sso_create_site_url(self, user_id, domain, locale=None):
        """Get SSO create site url for a particular user and domain.

        >>> yola.get_sso_create_site_url('user_id', 'example.com')
        """
        params = {'domain': domain, 'locale': locale}
        return self.get(
            self._user_path(
                user_id, 'sso_url_create_site'), params=params).json()['url']

    def get_sso_open_site_url(self, user_id, site_id=None, locale=None):
        """Get SSO open site url for a particular user.

        >>> yola.get_sso_open_site_url('user_id')
        """
        return self.get(
            self._user_path(user_id, 'sso_url_open_site'), params={
                'site_id': site_id,
                'locale': locale
            }).json()['url']

    def get_sso_url(self, user_id, site_id=None,
                    destination='editor', locale=None):
        """Get SSO url for a particular ws user
        >>> yola.get_sso_url('user_id')
        """
        return self.get(
            self._user_path(user_id, 'sso-url'), params={
                'site_id': site_id,
                'destination': destination,
                'locale': locale,
            }).json()['url']

    def get_user_wsites(self, user_id):
        return self.get(self._user_path(user_id, 'sites')).json()

    def set_site_url(self, user_id, site_url):
        """Set new site url for a particular user.

        >>> yola.set_site_url('user_id', 'https://new-domain.com')
        """
        return self.post(
            self._user_path(user_id, 'set-site-url'),
            json={'site_url': site_url}
        ).json()

    def _user_path(self, *parts):
        path = '/'.join(['users'] + list(parts))
        return '/%s/' % path



from yolapy.services import Yola


class Partner(object):

    """Yola Partner service model.

    Example use:
    ```
    > from yolapy.models import Partner
    > Partner.me().available_subscription_types
    ["wl_lite", "wl_premium", "wl_basic", "wl_ecommerce_basic"]
    ```
    """

    def __init__(self):
        self.client = Yola()
        self._partner = None

    def fetch_partner_data(self, partner_id):
        self._partner = self.client.get_partner(partner_id)

    @property
    def available_subscription_types(self):
        properties = self._partner['properties']
        return properties['available_subscription_types']

    @classmethod
    def me(cls):
        partner = cls()
        partner_id = partner.client.username
        partner.fetch_partner_data(partner_id)
        return partner



from yolapy.services import Yola


class Subscription(object):

    """Construct a Subscription.

    :param id: Str, 32 char ID
    :param created_at: Str, creation time
    :param updated_at: Str, last update time
    :param partner_id: Str, ID of the partner
    :param user_id: Str, 32 character ID of the owner
    :param status: Str
    :param term: Str
    :param sku: Str, SKU of the related product
    :param type: Str,
    :param start_date: Str
    :param expiry_date: Str
    :param billing_date: Str
    :param deprovision_date: Str
    :param auto_renew: Bool
    :param properties: Dict

    :return: Subscription
    :rtype: yolapy.models.Subscription

    """

    _fields = (
        'id', 'created_at', 'updated_at', 'partner_id', 'user_id', 'status',
        'term', 'sku', 'type', 'start_date', 'expiry_date', 'billing_date',
        'deprovision_date', 'auto_renew', 'properties')

    def __init__(self, **fields):
        self.client = Yola()
        for field_name in self._fields:
            setattr(self, field_name, fields.get(field_name))

    @classmethod
    def list(cls, **kwargs):
        """Return a filtered list of Subscriptions.

        Usage::

           >>> from yolapy.models import Subscription
           >>> user_id = 'abcdef1234567890abcdef1234567890'
           >>> user_subs = Subscription.list(user_id=user_id)
           >>> print user_subs[0]
           <yolapy.models.subscription.Subscription object>
           >>> print user_subs[0].user_id
           u'abcdef1234567890abcdef1234567890'
        """
        response = Yola().list_subscriptions(**kwargs)
        return [cls(**sub) for sub in response['results']]



from unittest import TestCase

from mock import patch

from yolapy.constants import SiteStates
from yolapy.models import Site


class SiteTestCase(TestCase):
    def setUp(self):
        yola_patcher = patch('yolapy.models.site.Yola')
        self.yola = yola_patcher.start().return_value
        self.addCleanup(yola_patcher.stop)


class TestSite(SiteTestCase):
    def test_sets_init_kwargs_as_attributes(self):
        site = Site(name='my site', id='123')
        self.assertEqual(site.name, 'my site')
        self.assertEqual(site.id, '123')


class TestSiteGet(SiteTestCase):
    """Site.get"""

    def test_instantiates_site_with_attrs_from_service(self):
        self.yola.get_site.return_value = {'name': 'site name', 'id': '456'}

        site = Site.get('456')

        self.yola.get_site.assert_called_once_with('456')
        self.assertEqual(site.name, 'site name')
        self.assertEqual(site.id, '456')


class TestSiteList(SiteTestCase):
    """Site.list"""

    def setUp(self):
        super(TestSiteList, self).setUp()

        self.yola.list_sites.return_value = {'results': [
            {'id': '1'}, {'id': '2'},
        ]}

        self.sites = Site.list(user_id='user123')
        _, self.service_kwargs = self.yola.list_sites.call_args

    def test_instantiates_list_of_sites_with_attrs_from_service(self):
        self.assertEqual(len(self.sites), 2)
        self.assertIn(Site(id='1'), self.sites)
        self.assertIn(Site(id='2'), self.sites)

    def test_filters_sites_by_passed_fields(self):
        self.assertEqual(self.service_kwargs['user_id'], 'user123')

    def test_handles_pagination_automatically(self):
        self.assertIn('page', self.service_kwargs)


class TestSiteIsPublished(SiteTestCase):
    """Site.is_published"""

    def test_false_if_no_publishing_data(self):
        site = Site()
        self.assertFalse(site.is_published)

    def test_false_if_publishing_state_is_not_published(self):
        site = Site(publishing_data={'state': 999})
        self.assertFalse(site.is_published)

    def test_true_if_publishing_state_is_published(self):
        site = Site(publishing_data={'state': SiteStates.PUBLISHED})
        self.assertTrue(site.is_published)



class SiteResourceMixin(object):
    """Methods for managing Site resources."""

    def list_sites(self, **options):
        """Return paginated list of sites.

        >>> yola.list_sites()
        {
            'count': 999,
            'previous': None,
            'next': 'https://wl.qa.yola.net/pr/sites/?page=2',
            'results': [{...}, {...}, ...]
        }

        You can pass ``page_size`` and ``page`` as keyword arguments:

        >>> yola.list_sites(page_size=50, page=2)

        You can also pass filters and odering options as keyword arguments.
        See https://wl.qa.yola.net/sites/ for available options.
        """
        return self.get(self._site_path(), params=options).json()

    def get_site(self, site_id):
        """Return details for a particular site.

        >>> site = yola.get_site('site_id')
        >>> site['name']
        'My Site'
        """
        return self.get(self._site_path(site_id)).json()

    def disable_site(self, site_id):
        """Disable a site.

        A disabled site can be edited but can't be published.

        >>> yola.disable_site('site_id')
        """
        self.post(self._site_path(site_id, 'disable'))

    def enable_site(self, site_id):
        """Enable a site.

        >>> yola.enable_site('site_id')
        """
        self.post(self._site_path(site_id, 'enable'))

    def change_site_owner(self, site_id, new_user_id):
        """Change site owner.

        >>> yola.change_site_owner('site_id', 'new_user_id')
        """
        data = {'id': new_user_id}
        self.post(self._site_path(site_id, 'change_owner'), data=data)

    def change_site_domain(self, site_id, new_domain):
        """Change site's domain.

        >>> yola.change_site_domain('site_id', 'newdomain.com')
        """
        data = {'partner_domain': new_domain}
        self.put(self._site_path(site_id, 'partner_domain'), json=data)

    def delete_site(self, site_id):
        """Delete a site.

        >>> yola.get_site('site_id')['deleted_at']
        None
        >>> yola.delete_site('site_id')
        >>> yola.get_site('site_id')['deleted_at']
        '2015-08-27T10:00:07'
        """
        self.delete(self._site_path(site_id))

    def undelete_site(self, site_id):
        """Un-delete a site.

        >>> yola.get_site('site_id')['deleted_at']
        '2015-08-27T10:00:07'
        >>> yola.undelete_site('site_id')
        >>> yola.get_site('site_id')['deleted_at']
        None
        """
        self.post(self._site_path(site_id, 'undelete'))

    def _site_path(self, *parts):
        path = '/'.join(['sites'] + list(parts))
        return '/%s/' % path

    def _create_site(self, **attributes):
        return self.post(self._site_path(), json=attributes).json()



class WLTaskResourceMixin(object):
    """Methods for managing WL Task resources."""

    def complete_wl_task(self, wl_task_id):
        """Complete WL Task.

        >>> yola.complete_wl_task('ae24b9c8048d4b0d9734263c7165bb79')
        {
            'id': 'ae24b9c8048d4b0d9734263c7165bb79,
            'status': 'completed,
            'details': {},
        }
        """
        return self.post('/wl-tasks/{}/complete/'.format(wl_task_id)).json()

    def fail_wl_task(self, wl_task_id, reason):
        """Fail WL Task.

        >>> yola.fail_wl_task(
        >>>    'ae24b9c8048d4b0d9734263c7165bb79', 'Unexpected Error'
        >>> )
        {
            'id': 'ae24b9c8048d4b0d9734263c7165bb79,
            'status': 'failed,
            'details': {'error': 'Unexpected Error'},
        }
        """
        return self.post(
            '/wl-tasks/{}/fail/'.format(wl_task_id),
            json={'reason': reason}
        ).json()



from tests.test_integration.helpers import (
    create_site, create_user_with_subscription)
from tests.test_integration.test_case import YolaServiceTestCase
from yolapy.models import Site


class TestYolaSite(YolaServiceTestCase):
    """Yola: Site resource"""

    @classmethod
    def setUpClass(cls):
        super(TestYolaSite, cls).setUpClass()
        cls.site = create_site(cls.service)

    def test_list_sites(self):
        sites = self.service.list_sites(
            user_id=self.site['user_id'])['results']
        site_ids = [s['id'] for s in sites]
        self.assertIn(self.site['id'], site_ids)

    def test_get_site(self):
        exptected_site_id = self.site['id']
        site = self.service.get_site(exptected_site_id)
        self.assertEqual(site['id'], exptected_site_id)

    def test_disable_and_enable_site(self):
        self.service.disable_site(self.site['id'])
        self.service.enable_site(self.site['id'])
        # no way to verify? just checking it doesn't raise for now...

    def test_change_site_owner(self):
        new_user = create_user_with_subscription(self.service)
        self.service.change_site_owner(self.site['id'], new_user['id'])

        site = self.service.get_site(self.site['id'])
        self.assertEqual(site['user_id'], new_user['id'])
        self.site['user_id'] = new_user['id']

    def test_delete_and_undelete_site(self):
        self.service.delete_site(self.site['id'])
        site = self.service.get_site(self.site['id'])
        self.assertTrue(site['deleted_at'])

        self.service.undelete_site(self.site['id'])
        site = self.service.get_site(self.site['id'])
        self.assertFalse(site['deleted_at'])

    def test_list_sites_using_model(self):
        sites = Site.list()
        site_ids = [s.id for s in sites]
        self.assertIn(self.site['id'], site_ids)



# setup.py needs to import this without worrying about required packages being
# installed. So if you add imports here, you may need to do something like:
# https://github.com/mitsuhiko/click/blob/da080dd2de1a116fc5789ffdc1ec6ffb864f2044/setup.py#L1-L11
__version__ = '0.8.0'



from unittest import TestCase

from mock import patch

from yolapy.models.user import User


class UserTestCase(TestCase):

    def setUp(self):
        self.client_patcher = patch('yolapy.models.user.Yola')
        self.addCleanup(self.client_patcher.stop)
        self.client = self.client_patcher.start().return_value
        self.client.username = 'WL_DEFAULT'
        User._client = None


class TestUserClient(UserTestCase):

    def test_is_a_yola_client(self):
        self.assertEqual(User().get_client(), self.client)


class TestUserGet(UserTestCase):
    """User.get"""
    def setUp(self):
        super(TestUserGet, self).setUp()
        self.client.get_user.return_value = {
            'id': '123',
            'name': 'Firstname',
            'surname': 'Lastname',
            'email': 'email@example.com',
            'active': True,
            'signup_date': '2016-01-01',
            'deleted': '2016-02-01',
            'partner_id': 'PARTNER',
            'preferences': {'currency': 'USD'},
            'preferred_greeting': 'hi',
        }

        self.user = User.get('123')

    def test_instantiates_user_with_attrs_from_service(self):
        self.client.get_user.assert_called_once_with('123')
        self.assertEqual(self.user.id, '123')
        self.assertEqual(self.user.active, True)
        self.assertEqual(self.user.deleted, '2016-02-01')
        self.assertEqual(self.user.signup_date, '2016-01-01')
        self.assertEqual(self.user.partner_id, 'PARTNER')
        self.assertEqual(self.user.name, 'Firstname')
        self.assertEqual(self.user.surname, 'Lastname')
        self.assertEqual(self.user.email, 'email@example.com')
        self.assertEqual(self.user.name, 'Firstname')
        self.assertDictEqual(self.user.preferences, {'currency': 'USD'})

    def test_undeclared_fields_are_not_added_as_an_attribute(self):
        with self.assertRaises(AttributeError):
            self.user.greeting


class TestUserSave(UserTestCase):

    def test_uses_service_to_create_a_new_user(self):
        user_attrs = {
            'name': 'First',
            'surname': 'Name',
            'email': 'email@example.com',
            'partner_id': 'WL_PARTNER',
            'preferences': {'locale': 'en'},
        }
        User.create(**user_attrs)
        create_user_call = self.client.create_user.call_args[1]
        self.assertEqual(user_attrs, create_user_call)


class TestUserPartnerId(UserTestCase):

    def test_uses_client_config_as_default_partner_id(self):
        self.assertEqual(User().partner_id, 'WL_DEFAULT')

    def test_can_be_set_via_init(self):
        self.assertEqual(User(partner_id='WL_DOG').partner_id, 'WL_DOG')

    def test_can_be_set_via_attribute(self):
        user = User()
        user.partner_id = 'WL_CAT'
        self.assertEqual(user.partner_id, 'WL_CAT')


class TestUserUpdate(UserTestCase):

    def test_updates_all_attributes_passed_in(self):
        user_attrs = {'name': 'Nameski', 'email': 'emailz@example.com'}
        u = User()
        u.update(**user_attrs)
        self.assertEqual(u.name, 'Nameski')
        self.assertEqual(u.email, 'emailz@example.com')



from demands.pagination import PaginatedResults

from yolapy.services import Yola


class SiteImport(object):
    """Represents a SiteImport resource from the Yola API"""

    def __init__(self, **kwargs):
        for key, val in kwargs.items():
            setattr(self, key, val)

    @classmethod
    def create(cls, url, user_id):
        """Create a SiteImport with the Yola API"""
        return cls(**Yola().create_site_import(url=url, user_id=user_id))

    @classmethod
    def get(cls, id):
        """Get a SiteImport from the Yola API"""
        return cls(**Yola().get_site_import(id))

    @classmethod
    def list(cls, **filters):
        """Get a list of SiteImports from the Yola API"""
        site_imports = PaginatedResults(
            Yola().list_site_imports, kwargs=filters)
        return [cls(**site_import) for site_import in site_imports]



from tests.test_integration.helpers import (
    create_user_with_subscription)
from tests.test_integration.test_case import YolaServiceTestCase


class TestYolaSiteImport(YolaServiceTestCase):
    """Yola: Site Import resource"""

    @classmethod
    def setUpClass(cls):
        super(TestYolaSiteImport, cls).setUpClass()
        cls.user = create_user_with_subscription(
            cls.service, **{'name': 'Kendrick Lamar'})
        cls.site_import = cls.service.create_site_import(
            **{'url': 'http://yola.com', 'user_id': cls.user['id']})

    def test_create_site_import(self):
        self.assertTrue(self.site_import)

    def test_get_site_import(self):
        self.assertEqual(
            self.service.get_site_import(self.site_import['id']),
            self.site_import)

    def test_get_user_site_imports(self):
        self.assertEqual(
            self.service.list_site_imports(user_id=self.user['id'])['results'],
            [self.site_import])



from yolapy.services import Yola


class User(object):
    """Yola User - a service model that uses the yola client for persistence.

    Example use:
    ```
    from yolapy.models import User as YolaUser

    yola_user = YolaUser(
        email='test@example.com',
        name='Jane',
        surname='Doe',
        partner_id='WL_PARTNER',
        preferences={'locale': 'en'})

    yola_user.save()
    ```
    """

    _client = None
    _fields = ('active', 'deleted', 'email', 'id', 'name', 'partner_id',
               'preferences', 'signup_date', 'surname')

    def __init__(self, **kwargs):
        """Construct a Yola User.

        The data in a User instance does not persist until it has been saved.


        :param active: Bool, toggled ``False`` when user is deactivated
        :param deleted: Timestamp, date the user was deleted
        :param email: Str, user's email address
        :param id: Str, user's 32 character ID
        :param name: Str, user's first name
        :param partner_id: Str, yola partner responsible the user
        :param preferences: Dict, user's preferences
        :param signup_date: Timestamp, date the user signed up
        :param surname: Str, user's second name

        :return: :class:`User <User>` object
        :rtype: yolapy.users.models.User

        """

        for field_name in self._fields:
            setattr(self, field_name, kwargs.get(field_name))

        self.partner_id = self.partner_id or self.get_client().username
        self.active = self.active or False
        self.preferences = self.preferences or {}

    @classmethod
    def get_client(cls):
        if not cls._client:
            cls._client = Yola()
        return cls._client

    @classmethod
    def get(cls, user_id):
        """Get a user from the Yola API."""
        user_attributes = cls.get_client().get_user(user_id)
        return cls(**user_attributes)

    @classmethod
    def create(cls, **kwargs):
        user = cls.get_client().create_user(**kwargs)
        return cls(**user)

    def update(self, **attributes):
        """Update all keyword arguments to update instance attributes."""
        for attr, val in attributes.items():
            setattr(self, attr, val)



from unittest import TestCase
from mock import patch

from yolapy.models import Partner


class PartnerTestCase(TestCase):

    def setUp(self):
        self.client_patcher = patch('yolapy.models.partner.Yola')
        self.addCleanup(self.client_patcher.stop)
        self.client = self.client_patcher.start().return_value
        self.client.username = 'WL_DEFAULT'


class TestPartnerMe(PartnerTestCase):

    """Partner.me()."""

    def test_has_available_subscription_types(self):
        Partner.me().available_subscription_types

    @patch.object(Partner, 'fetch_partner_data')
    def test_fetches_data_using_the_client_username(self, fetch):
        Partner.me()
        fetch.assert_called_once_with('WL_DEFAULT')



from unittest import TestCase

from mock import patch
from yolapy import configuration
from yolapy.configuration import configure, get_config


@patch.dict('yolapy.configuration.config', clear=True)
class TestConfiguration(TestCase):

    def test_sets_a_simple_value(self):
        configure(FOO='bar')
        self.assertEqual(configuration.config['FOO'], 'bar')

    def test_gets_a_config(self):
        configuration.config = {'foo': 'bar'}
        self.assertEqual(get_config('foo'), 'bar')

    def test_raises_if_config_is_missing(self):
        configuration.config = {'foo': 'bar'}
        self.assertRaises(KeyError, get_config, 'baz')

    def test_uses_default_if_config_is_missing(self):
        configuration.config = {'foo': 'bar'}
        self.assertEqual(get_config('baz', 'qux'), 'qux')



from demands.pagination import PaginatedResults

from yolapy.services import Yola


class Site(object):
    """Represents a Site resource on the Yola API."""

    def __init__(self, **kwargs):
        self._fields = kwargs
        for key, val in kwargs.items():
            setattr(self, key, val)
        self.publishing_data = kwargs.get('publishing_data') or {}

    def __eq__(self, other):
        return self._fields == other._fields

    @classmethod
    def get(cls, site_id):
        """Get a site from the Yola API."""
        site_attributes = Yola().get_site(site_id)
        return cls(**site_attributes)

    @classmethod
    def list(cls, **filters):
        """Get a list of sites from the Yola API."""
        sites = PaginatedResults(Yola().list_sites, kwargs=filters)
        return [Site(**s) for s in sites]



from unittest import TestCase

from tests.test_integration import test_settings as settings
from yolapy.configuration import configure


# we have to configure before importing
configure(auth=settings.auth, url=settings.url, verify_ssl=False)
from yolapy.services import Yola


class YolaServiceTestCase(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.service = Yola()



from time import time
from uuid import uuid4


def create_partner(service, **custom_attrs):
        # can't use a uuid because of 30 char limit:
        unique_id = str(time()).replace('.', '')

        attrs = {
            'id': 'WL_TEST_%s' % unique_id,
            'name': 'TEST',
            'parent_partner_id': service.username,
            'properties': {'website': 'example.com'},
        }
        attrs.update(custom_attrs)

        return service.create_partner(**attrs)


def create_user(service, **custom_attrs):
    """Create test user with default attributes.

    Override default attributes by passing keyword arguments.
    """
    attrs = {
        'email': 'test+%s@yola.com' % uuid4().hex,
        'name': 'John',
        'surname': 'Smith',
        'partner_id': service.username,
        'preferences': {},
    }
    attrs.update(custom_attrs)
    return service.create_user(**attrs)


def create_user_with_subscription(service, **custom_attrs):
    user = create_user(service, **custom_attrs)
    service.create_subscription('wl_basic', user['id'], {})
    return user


def create_site(service, user_id=None):
    """Create test site for given user_id.

    If no user id is provided, a test user will be created.
    """
    if not user_id:
        user_id = create_user_with_subscription(service)['id']

    serialized_site = {
        'id': uuid4().hex,
        'cryptoKey': uuid4().hex,
        'user_id': user_id,
        'owner_id': user_id,
        'name': 'My Test Site',
        'version': 62,
        'locale': 'en',
        'properties': {},
        'facebook_page_id': None,
        'site_type': '1',
        'auth_user': None,
        'auth_pass': None,
        'site_template': 'default',
        'base_template': 'SuperFlat_v2',
        'navigation': [],
    }

    site_resource = service._create_site(**serialized_site)
    site_resource['user_id'] = user_id
    return site_resource



from yolapy.models.user import User  # noqa
from yolapy.models.partner import Partner  # noqa
from yolapy.models.site import Site  # noqa
from yolapy.models.siteimport import SiteImport  # noqa
from yolapy.models.subscription import Subscription  # noqa



class CnameZoneMixin(object):
    """Methods for managing Cname Zones."""
    def create_cname_zone(self, hostname, partner_id):
        return self.post(
            '/cname-dns-zones/',
            json={'partner_id': partner_id, 'hostname': hostname}).json()



from tests.test_integration.test_case import YolaServiceTestCase

from tests.test_integration.helpers import create_partner, create_user


class TestYolaSubscription(YolaServiceTestCase):
    """Yola: Subscription resource"""

    @classmethod
    def setUpClass(cls):
        super(TestYolaSubscription, cls).setUpClass()
        cls.partner = create_partner(cls.service, properties={
            'website': 'example.com',
            'available_subscription_types': ['wl_basic', 'wl_lite']
        })

        user = create_user(cls.service, partner_id=cls.partner['id'])
        cls.subscription = cls.service.create_subscription(
            'wl_basic', user['id'], {'foo': 'bar'})

    def test_create_subscriptions(self):
        self.assertEqual(self.subscription['status'], 'active')

    def test_list_subscriptions(self):
        subs = self.service.list_subscriptions()['results']
        self.assertTrue(len(subs) > 0)

    def test_get_subscription(self):
        sub = self.service.get_subscription(self.subscription['id'])
        self.assertEqual(sub['id'], self.subscription['id'])

    def test_cancel_and_reactivate_subscription(self):
        self.assertEqual(self.subscription['status'], 'active')

        self.service.cancel_subscription(self.subscription['id'], 'testing')
        cancelled_sub = self.service.get_subscription(self.subscription['id'])
        self.assertEqual(cancelled_sub['status'], 'canceled')

        self.service.reactivate_subscription(cancelled_sub['id'], 'testing')
        reactivated_sub = self.service.get_subscription(cancelled_sub['id'])
        self.assertEqual(reactivated_sub['status'], 'active')

    def test_can_activate_a_trial_subscription(self):
        user = create_user(self.service, partner_id=self.partner['id'])
        sub = self.service.create_subscription(
            'wl_basic', user['id'], {'trial': True})
        self.assertTrue(sub['properties']['trial'])

        activated_sub = self.service.activate_trial_subscription(sub['id'])
        self.assertFalse(activated_sub['properties'].get('trial'))

    def test_can_change_subscription_type(self):
        user = create_user(self.service, partner_id=self.partner['id'])
        sub = self.service.create_subscription(
            'wl_basic', user['id'], {'trial': True})
        self.assertEqual(sub['type'], 'wl_basic')

        modified_sub = self.service.change_subscription_type(
            sub['id'], 'wl_lite')
        self.assertEqual(modified_sub['type'], 'wl_lite')



from uuid import uuid4

from demands import HTTPServiceError

from tests.test_integration.test_case import YolaServiceTestCase


class TestYolaSubscription(YolaServiceTestCase):
    """Yola: Subscription resource"""

    def test_cname_zone_creation_api_is_called(self):
        self.assertRaisesHttpServiceError(
            400, '["zone is already taken by different user"]',
            self.service.create_cname_zone,
            '{}.example.org'.format(uuid4().hex),
            'WL_BOTTOMLINE_YOLA'
        )

    def assertRaisesHttpServiceError(
            self, status_code, msg, func, *args, **kwargs):
        try:
            func(*args, **kwargs)
            self.assertFail()
        except HTTPServiceError, exc:
            self.assertEqual(exc.response.status_code, status_code)
            self.assertEqual(
                exc.response.content,
                '["zone is already taken by different user"]'
            )



class SiteImportResourceMixin(object):
    """Yola API methods for SiteImports"""

    _path = '/pr/jobs/sites/'

    def create_site_import(self, **data):
        """Create a SiteImport"""
        return self.post(
            self._path, json=data).json()

    def get_site_import(self, id):
        """Return data for a specific SiteImport"""
        return self.get('{url}{id}/'.format(url=self._path, id=id)).json()

    def list_site_imports(self, **options):
        """Return a paginated list of SiteImports"""
        return self.get(self._path, params=options).json()



from unittest import TestCase

from mock import patch

from yolapy.models import Subscription


class SubscriptionTestCase(TestCase):
    fake_subscription_data = {
        'id': 'fake-sub-id-abcdef1234567890abcd',
        'created_at': '2016-05-31 13:11:36',
        'updated_at': '2016-05-31 13:11:38',
        'auto_renew': False,
        'billing_date': '2000-01-01',
        'deprovision_date': '2000-01-01',
        'expiry_date': '2000-01-01',
        'start_date': '2000-01-01',
        'status': 'active',
        'sku': '123',
        'type': 'wl_basic',
        'user_id': 'fake-user-id-abcdef1234567890abc',
        'partner_id': 'partner-id-foo123',
        'term': 'P1M',
        'properties': {}
    }

    def setUp(self):
        self._mock_client()
        self._stub_list_subscriptions()

    def _mock_client(self):
        self.client_patcher = patch('yolapy.models.subscription.Yola')
        self.addCleanup(self.client_patcher.stop)
        self.client = self.client_patcher.start().return_value

    def _stub_list_subscriptions(self):
        self.client.list_subscriptions.return_value = {
            'results': [self.fake_subscription_data],
        }


class TestSubscription(SubscriptionTestCase):
    def test_has_expected_attributes(self):
        sub = Subscription(**self.fake_subscription_data)
        for key, value in self.fake_subscription_data.items():
            self.assertEqual(getattr(sub, key), value)


class SubscriptionList(SubscriptionTestCase):

    """Subscription.list()."""

    def test_fetches_subs_by_passing_arguments_to_client(self):
        Subscription.list(foo='bar', user_id='user-id')
        self.client.list_subscriptions.assert_called_with(
            foo='bar', user_id='user-id')

    def test_creates_subs_using_response(self):
        subs = Subscription.list()
        self.assertEqual(len(subs), 1)
        subscription = subs[0]
        self.assertEqual(subscription.id, 'fake-sub-id-abcdef1234567890abcd')



from tests.test_integration.helpers import create_partner
from tests.test_integration.test_case import YolaServiceTestCase


class TestYolaPartners(YolaServiceTestCase):
    """Yola: Partner resource"""

    @classmethod
    def setUpClass(cls):
        super(TestYolaPartners, cls).setUpClass()
        cls.partner = create_partner(cls.service)
        cls.partner_id = cls.partner['id']
        # There's no way to really delete the test partners we're creating :(

    def test_create_partner(self):
        self.assertEqual(self.partner['properties']['website'], 'example.com')

    def test_get_partner(self):
        partner = self.service.get_partner(self.partner_id)
        self.assertEqual(partner['name'], 'TEST')

    def test_delete_partner(self):
        partner = create_partner(self.service)
        self.service.delete_partner(partner['id'])
        partner = self.service.get_partner(partner['id'])
        self.assertFalse(partner['is_active'])

    def test_update_partner(self):
        partner = create_partner(self.service, name='Original Name')
        partner = self.service.update_partner(partner['id'], name='New Name')
        self.assertEqual(partner['name'], 'New Name')

    def test_list_partners(self):
        response = self.service.list_partners(page_size=1)
        self.assertEqual(len(response['results']), 1)



class CampaignResourceMixin(object):
    """Methods for managing Campaign resources."""

    def list_campaigns(self, site_id, **options):
        """Return list of campaigns for a site."""
        return self.get(self._campaign_path(site_id), params=options).json()

    def get_campaign(self, site_id, campaign_id):
        """Return details for a particular campaign.

        >>> campaign = yola.get_campaign('site_id', 'campaign_id')
        >>> campaign['name']
        'My Campaign'
        """
        return self.get(self._campaign_path(site_id, campaign_id)).json()

    def delete_campaign(self, site_id, campaign_id):
        """Delete a campaign."""
        self.delete(self._campaign_path(site_id, campaign_id))

    def _campaign_path(self, site_id, *parts):
        path = '/'.join(['sites', site_id, 'campaigns'] + list(parts))
        return '/%s/' % path



from demands import HTTPServiceClient

from yolapy.configuration import config as defaults
from yolapy.resources import (
    campaign,
    cname_zone,
    partner,
    site,
    siteimport,
    subscription,
    user,
    wl_task,
)


class Yola(
    HTTPServiceClient,
    campaign.CampaignResourceMixin,
    cname_zone.CnameZoneMixin,
    partner.PartnerResourceMixin,
    site.SiteResourceMixin,
    siteimport.SiteImportResourceMixin,
    subscription.SubscriptionResourceMixin,
    user.UserResourceMixin,
    wl_task.WLTaskResourceMixin,
):
    """Client for Yola's API.

    If using yolapy.configuration::

        configure(
            url='https://wl.yola.net/',
            auth=('username', 'password'))
        yola = Yola()
        yola.get_user('user_id')

    Or configured manually::

        yola = Yola(
            url='https://wl.yola.net/',
            auth=('username', 'password'))
        yola.get_user('user_id')

    When appropriate, successful responses will return parsed json objects.

    Failures will raise instances of ``demands.HTTPServiceError``.
    """

    def __init__(self, **kwargs):
        """Initialize with optional headers.

        Auth and url defaults are pulled from yolapy.configuration.

        Passed arguments will override configuration::

            Yola(headers={'Header-Name': 'value'})
        """
        config = {}
        config.update(defaults)
        config.update(kwargs)
        assert(config['url'])
        assert(config['auth'])
        self.username = config['auth'][0]
        super(Yola, self).__init__(**config)



class PartnerResourceMixin(object):
    """Methods for managing Partner resources."""

    def create_partner(self, **attributes):
        """Create a partner.

        >>> partner = yola.create_partner(
            id='COMPANY_ID',
            name='Company Name',
            parent_partner_id='PARENT_ID',
            properties={'name': 'value'})
        >>> partner['name']
        'Company Name'

        See https://wl.qa.yola.net/partners/ for available properties.
        """
        return self.post(self._partner_path(), json=attributes).json()

    def get_partner(self, partner_id):
        """Return details for a particular partner.

        >>> partner = yola.get_partner('PARTNER_ID')
        >>> partner['name']
        'Company Name'
        """
        return self.get(self._partner_path(partner_id)).json()

    def delete_partner(self, partner_id):
        """Delete a partner.

        >>> yola.delete_partner('PARTNER_ID')
        """
        self.delete(self._partner_path(partner_id))

    def update_partner(self, partner_id, **attributes):
        """Update a partner.

        >>> yola.update_partner('PARTNER_ID', name='New name')
        """
        return self.patch(
            self._partner_path(partner_id), json=attributes).json()

    def list_partners(self, **options):
        """Return paginated list of partners.

        >>> yola.list_partners()
        {
            'count': 999,
            'previous': None,
            'next': 'https://wl.qa.yola.net/pr/partners/?page=2',
            'results': [{...}, {...}, ...]
        }

        You can pass ``page_size`` and ``page`` as keyword arguments:

        >>> yola.list_partners(page_size=50, page=2)
        """
        return self.get(self._partner_path(), params=options).json()

    def _partner_path(self, *parts):
        path = '/'.join(['partners'] + list(parts))
        return '/%s/' % path



"""Configuration.

Yolapy.configuration provides a key-value store used by the Yola client.

Data is stored here in the module, benefits include:

 * Configuration is decoupled from application logic.

 * When instantiating multiple service models, each contains its own client.
   This module allows for configuration to happen once then consumed multiple
   times by each client.

See README for example of use.

"""
config = {}
_missing = object()


def configure(**kwargs):
    """Save all keyword arguments as (key=value) configuration."""
    config.update(kwargs)


def get_config(key, default=_missing):
    """Lookup the value of a configuration key using an optional default."""
    value = config.get(key, default)
    if value == _missing:
        raise KeyError('%s is not configured' % key)
    return value



from setuptools import find_packages, setup

import yolapy


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('CHANGELOG.rst') as changelog_file:
    changelog = changelog_file.read()

setup(
    name='yolapy',
    version=yolapy.__version__,
    description='Python client for the Yola API',
    long_description='%s\n\n%s' % (readme, changelog),
    author='Yola',
    author_email='engineers@yola.com',
    license='MIT (Expat)',
    url='https://github.com/yola/yolapy',
    packages=find_packages(),
    test_suite='nose.collector',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Topic :: Internet :: WWW/HTTP',
    ],
    install_requires=[
        'demands >= 4.0.0, < 6.0.0',
    ],
    tests_require=[
        'nose >= 1.3.0, < 1.4.0',
    ],
)



class SubscriptionResourceMixin(object):
    """Methods for managing Subscription resources."""

    def list_subscriptions(self, **options):
        """Return paginated list of subscriptions.

        >>> yola.list_subscriptions()
        {
            'count': 999,
            'previous': None,
            'next': 'https://wl.qa.yola.net/pr/subscriptions/?page=2',
            'results': [{...}, {...}, ...]
        }

        You can pass ``page_size`` and ``page`` as keyword arguments:

        >>> yola.list_subscriptions(page_size=50, page=2)

        You can also pass filters and odering options as keyword arguments.
        See https://wl.qa.yola.net/subscriptions/ for available options.
        """
        return self.get(self._subscription_path(), params=options).json()

    def get_subscription(self, subscription_id):
        """Return details for a particular subscription.

        >>> subscription = yola.get_subscription('subscription_id')
        >>> subscription['name']
        'My Subscription'
        """
        return self.get(self._subscription_path(subscription_id)).json()

    def change_subscription_type(self, subscription_id, new_type):
        """Change subscription type.

        >>> yola.change_subscription_type('subscription_id', 'new_type')

        See https://wl.qa.yola.net/subscriptions/ for available types.
        """
        return self.post(
            self._subscription_path(subscription_id, 'change_type'),
            json={'new_type': new_type}).json()

    def cancel_subscription(self, subscription_id, reason):
        """Cancel active subscription.

        >>> yola.cancel_subscription('subscription_id', 'some reason')
        """
        return self.post(
            self._subscription_path(subscription_id, 'cancel'),
            json={'reason': reason}).json()

    def reactivate_subscription(self, subscription_id, reason):
        """Re-activate a cancelled subscription.

        >>> yola.reactivate_subscription('subscription_id', 'some reason')
        """
        return self.post(
            self._subscription_path(subscription_id, 'reactivate'),
            json={'reason': reason}).json()

    def activate_trial_subscription(self, subscription_id):
        """Convert trial subscription to active.

        >>> yola.activate_trial_subscription('subscription_id')
        """
        return self.post(
            self._subscription_path(subscription_id, 'remove_trial')).json()

    def create_subscription(self, subscription_type, user_id, properties):
        """Create a new subscription.

        >>> subscription_type = 'wl_basic'
        >>> user_id = 'abcdef0123456789abcdef0123456789'
        >>> properties = {...}
        >>> yola.create_subscription(subscription_type, user_id, properties)

        See https://wl.qa.yola.net/subscriptions/ for available types and
        properties.
        """
        data = {
            'properties': properties,
            'type': subscription_type,
            'user_id': user_id,
        }
        return self.post(self._subscription_path(), json=data).json()

    def _subscription_path(self, *parts):
        path = '/'.join(['subscriptions'] + list(parts))
        return '/%s/' % path
